
BASIC_FORM_CSS = """\
form.qp div.title {
    font-weight: bold;
}

div.qpform, div.submit {
    background: #ddd;
    float: left;
}

form.qp div.widget {
    padding: 1ex;
}

form.qp div.widget div.widget {
    padding: 0;
    margin-bottom: 0;
}

form.qp div.SubmitWidget {
    float: left
}

form.qp div.content {
    margin-left: 0.6em; 
}

form.qp div.content div.content {
    margin-left: 0; 
}

form.qp div.error {
    color: #c30;
    font-size: small;
    margin-top: .1em;
}

form.qp div.hint {
    font-size: small;
    font-style: italic;
    margin-top: .1em;
}

form.qp div.errornotice {
    color: #c30;
    padding: 0.5em;
    margin: 0.5em;
    width: 40ex;
}
"""
