"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qp/demo/echo/__init__.py $
$Id: __init__.py 28044 2006-03-09 16:35:08Z dbinger $
"""
from qpy.compile import compile_qpy_files
compile_qpy_files(__path__[0])


