body { 
  padding: 0;
  margin: 0;
  background: grey;
  font-family: "Trebuchet MS", sans-serif;
}

div.body_top { 
  background: #888;
  color: white;
  padding-left: 1em;
  font-size: large;
}

div.body_middle { 
  background: #bbb;
  padding: 1em;
}

div.body_bottom { 
  clear: both;
  background: #888;
  color: white;
  padding-left: 1em;
}

a, a:link, a:hover, a:visited { 
  color: inherit;
}

form { 
  font-size: small;
}

form span.required { 
  display: none;
}

form div.error { 
  color: red;
}

th { 
  background: #ddd;
}

td { 
  background: #fff;
}

tr.current td { 
  background: lightblue;
}

ul.menu { 
  list-style-type: none;
  float: right;
  padding: 0;
  margin-top: -1em;
  margin-right: -1em;

}

ul.menu li { 
  padding-left: 1em;
  padding-right: 1em;
  background: #ddd;
  border: thin solid #bbb;
}

ul.menu li.selected { 
  background: lightblue;
}

dl.dict { 
  background: #ddd;
}

dl.dict dt { 
  font-weight: bold;
  padding-left: 1ex;
  font-size: small;
}

dl.dict dd { 
  white-space: pre;
  font-size: x-small;
}